<?php

namespace AG\HTTPRequester;

use AG\HTTPRequester\RequestContainer\RequestContainer;

class HTTPRequest
{
    const METHOD_POST = "POST";
    const METHOD_GET = "GET";
    const METHOD_PUT = "PUT";
    const METHOD_DELETE = "DELETE";

    private $login = NULL;
    private $password = NULL;
    private $url;
    private $method = NULL;
    private $headers = [];

    /**
     * HTTPRequest constructor.
     * @param null $login
     * @param null $password
     * @param $url
     * @param null $method
     * @param array $headers
     */
    public function __construct($url = NULL, $method = self::METHOD_GET, $headers = [], $login = NULL, $password = NULL)
    {
        $this->login = $login;
        $this->password = $password;
        $this->url = $url;
        $this->method = $method;
        $this->headers = $headers;
    }

    /**
     * @param mixed $login
     */
    public function setLogin($login)
    {
        $this->login = $login;
    }

    /**
     * @param mixed $password
     */
    public function setPassword($password)
    {
        $this->password = $password;
    }

    /**
     * @param mixed $url
     */
    public function setUrl($url)
    {
        $this->url = $url;
    }

    /**
     * @param mixed $method
     */
    public function setMethod($method)
    {
        $this->method = $method;
    }

    /**
     * @param mixed $headers
     */
    public function setHeaders($headers)
    {
        $this->headers = $headers;
    }

    /**
     * @param mixed $headers
     */
    public function addHeader($key, $value)
    {
        $this->headers[$key] = $value;
    }

    /**
     * @param mixed $headers
     */
    public function removeHeader($key)
    {
        unset($this->headers[$key]);
    }

    public function run(RequestContainer $content = NULL, &$curlInfos)
    {

        if ($this->method == NULL)
            $this->method = self::METHOD_GET;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $this->method);
        curl_setopt($ch, CURLOPT_URL, $this->url);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $this->headerFormatter($this->headers));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        if ($this->login != NULL) {
            curl_setopt($ch, CURLOPT_USERPWD, $this->login . ":" . $this->password);
            curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);

        }
        if ($content != NULL)
            curl_setopt($ch, CURLOPT_POSTFIELDS, $content->stringify());

        $response = curl_exec($ch);
        $curlInfos = curl_getinfo($ch);
        curl_close($ch);

        return $response;
    }

    public function performRequest($url, $method = NULL, RequestContainer $content = NULL, Array $headers = [], &$curlInfos = null)
    {
        $this->setUrl($url);
        $this->setMethod($method);
        $this->setHeaders($headers);
        return $this->run($content, $curlInfos);


    }

    private function headerFormatter($headers)
    {
        return array_map(function ($key, $value) {
            return $key . ": " . $value;
        }, array_keys($headers), array_values($headers));
    }
}